import os
from io import BytesIO

import requests
from PIL import Image, ImageDraw, ImageFont
from fuzzywuzzy import process

current_folder = os.path.dirname(os.path.abspath(__file__))


def get_closest_country_name(query):
    response = requests.get("https://restcountries.com/v3.1/all")
    if response.status_code == 200:
        country_names = [country['name']['common'] for country in response.json()]
        closest_match = process.extractOne(query, country_names)
        return closest_match[0]
    else:
        return None


def get_flag_image(country_name):
    # Create cache directory if it doesn't exist
    cache_dir = os.path.join(current_folder, "images/nation_cache")
    # Construct the file path for the cached image
    cache_file_path = os.path.join(cache_dir, f"{country_name}.png")

    # Check if the flag is already cached
    if os.path.exists(cache_file_path):
        return Image.open(cache_file_path)

    # Fetch the flag image from the API if not cached
    closest_country_name = get_closest_country_name(country_name)

    if closest_country_name:

        response = requests.get(f"https://restcountries.com/v3.1/name/{closest_country_name}")
        if response.status_code == 200:
            country_data = response.json()[0]
            flag_url = country_data['flags']['png']

            # Fetch the flag image from the URL
            image_response = requests.get(flag_url)
            if image_response.status_code == 200:
                image = Image.open(BytesIO(image_response.content))

                # Save the flag image to the cache
                image.save(cache_file_path)

                return image
    return None


def add_image_to_image(original_image, image, position, size=None, center=False):
    if size:
        image = image.resize(size)

    # Ensure the pasted image has an alpha channel
    if image.mode != 'RGBA':
        image = image.convert('RGBA')

    # Ensure the original image is in a mode that supports alpha (if necessary)
    if original_image.mode != 'RGBA':
        original_image = original_image.convert('RGBA')

    if center:
        # Calculate the centered position
        image_width, image_height = image.size
        position = (position[0] - image_width // 2, position[1] - image_height // 2)

    # Paste the image onto the original image
    original_image.paste(image, position, image)

    return original_image


def add_text_to_image(image, text, position, font_path, font_size, center=False):
    # Initialize ImageDraw object
    draw = ImageDraw.Draw(image)

    # Load the font
    font = ImageFont.truetype(font_path, font_size)

    if not center:
        # Add text to the image
        draw.text(position, text, font=font, fill=(73, 57, 15))
    else:
        # Calculate text bounding box
        text_bbox = draw.textbbox(position, text, font=font)

        # Calculate centered position
        text_width = text_bbox[2] - text_bbox[0]
        text_height = text_bbox[3] - text_bbox[1]

        centered_position = (position[0] - text_width // 2, position[1] - text_height // 2)

        # Add centered text to the image
        draw.text(centered_position, text, font=font, fill=(73, 57, 15))  # Black text

    return image


def get_card_stats_pos(stats: list = None):
    if not stats:
        stats = ["PAC", "SHO", "PAS", "DRI", "DEF", "PHY"]
    left_x = 180
    right_x = 520
    basic_y = 750
    y_space = 80

    stat_text_space = 80
    num_y_offset = 5

    stats_text_pos_dict = {
        stats[0]: (left_x, basic_y),
        stats[1]: (left_x, basic_y + y_space),
        stats[2]: (left_x, basic_y + y_space * 2),
        stats[3]: (right_x, basic_y),
        stats[4]: (right_x, basic_y + y_space),
        stats[5]: (right_x, basic_y + y_space * 2)
    }
    stats_num_pos_dict = {
        stats[0]: (left_x - stat_text_space, basic_y - num_y_offset),
        stats[1]: (left_x - stat_text_space, basic_y + y_space - num_y_offset),
        stats[2]: (left_x - stat_text_space, basic_y + y_space * 2 - num_y_offset),
        stats[3]: (right_x - stat_text_space, basic_y - num_y_offset),
        stats[4]: (right_x - stat_text_space, basic_y + y_space - num_y_offset),
        stats[5]: (right_x - stat_text_space, basic_y + y_space * 2 - num_y_offset)
    }
    return stats_text_pos_dict, stats_num_pos_dict


def fetch_all_teams(api_key="3"):
    url = f"https://www.thesportsdb.com/api/v1/json/{api_key}/search_all_teams.php?l="
    leagues = [
        "French Ligue 1",
        "French Ligue 2",
        "English Premier League",
        "English League Championship"
        "Spanish La Liga",
        "Italian Serie A",
        "German Bundesliga",
        "Dutch Eredivisie",
        "Portuguese Primeira Liga",
        "Belgian First Division A",
    ]

    teams = []

    for league in leagues:
        response = requests.get(url + league)
        if response.status_code == 200:
            league_data = response.json()
            if league_data['teams']:
                teams.extend(league_data['teams'])

    return teams


def get_important_stats(stat):
    important_stats = {
        "ST": ["SHO", "DRI", "PAC"],
        "RW": ["SHO", "DRI", "PAC"],
        "LW": ["SHO", "DRI", "PAC"],
        "LM": ["DRI", "PAC", "PAS"],
        "RM": ["DRI", "PAC", "PAS"],
        "CAM": ["SHO", "DRI", "PAC"],
        "CF": ["SHO", "DRI", "PAS"],
        "CM": ["SHO", "DRI", "PAS", "DEF"],
        "CDM": ["PHY", "DEF", "PAS"],
        "LB": ["DEF", "PAC", "PAS"],
        "RB": ["DEF", "PAC", "PAS"],
        "CB": ["DEF", "PHY", "PAC"]
    }
    if stat in important_stats:
        return important_stats[stat]
    else:
        return []


def get_closest_team_name(club_name, teams):
    team_names = [team['strTeam'] for team in teams]
    closest_match = process.extractOne(club_name, team_names)
    return closest_match[0]


def get_club_logo(club_name, api_key="3"):
    # Create cache directory if it doesn't exist
    cache_dir = os.path.join(current_folder, "images/club_logo_cache")
    os.makedirs(cache_dir, exist_ok=True)

    # Construct the file path for the cached image
    cache_file_path = os.path.join(cache_dir, f"{club_name.lower().replace(' ', '_')}.png")

    # Check if the logo is already cached
    if os.path.exists(cache_file_path):
        return Image.open(cache_file_path)

    # Fetch all teams and find the closest match
    teams = fetch_all_teams(api_key)
    closest_team_name = get_closest_team_name(club_name, teams)

    # Fetch the club logo from the API if not cached
    url = f"https://www.thesportsdb.com/api/v1/json/{api_key}/searchteams.php?t={closest_team_name}"
    response = requests.get(url)
    if response.status_code == 200:
        team_data = response.json()
        if team_data['teams']:
            if "strBadge" not in team_data['teams'][0]:
                logo_url = team_data['teams'][0]['strTeamBadge']
            else:
                logo_url = team_data['teams'][0]['strBadge']
            if logo_url:
                # Fetch the logo image from the URL
                image_response = requests.get(logo_url)
                if image_response.status_code == 200:
                    image = Image.open(BytesIO(image_response.content))

                    # Save the logo image to the cache
                    image.save(cache_file_path)

                    return image
    return None


def calculate_overall_ratings(stats, debug=False):
    coeff_multiplier = 1.022
    formulas = {
        "ST": lambda s: (coeff_multiplier * 0.40 * s["SHO"]) + (coeff_multiplier * 0.30 * s["PAC"]) + (
                coeff_multiplier * 0.20 * s["DRI"]) + (coeff_multiplier * 0.10 * s["PHY"]),
        "RW": lambda s: (coeff_multiplier * 0.35 * s["DRI"]) + (coeff_multiplier * 0.30 * s["PAC"]) + (
                coeff_multiplier * 0.25 * s["SHO"]) + (coeff_multiplier * 0.10 * s["PAS"]),
        "LW": lambda s: (coeff_multiplier * 0.35 * s["DRI"]) + (coeff_multiplier * 0.30 * s["PAC"]) + (
                coeff_multiplier * 0.25 * s["SHO"]) + (coeff_multiplier * 0.10 * s["PAS"]),
        "CF": lambda s: (coeff_multiplier * 0.40 * s["SHO"]) + (coeff_multiplier * 0.30 * s["DRI"]) + (
                coeff_multiplier * 0.20 * s["PAC"]) + (coeff_multiplier * 0.10 * s["PAS"]),
        "CAM": lambda s: (coeff_multiplier * 0.35 * s["DRI"]) + (coeff_multiplier * 0.35 * s["PAS"]) + (
                coeff_multiplier * 0.25 * s["SHO"]) + (coeff_multiplier * 0.05 * s["PAC"]),
        "CM": lambda s: (coeff_multiplier * 0.45 * s["PAS"]) + (coeff_multiplier * 0.30 * s["DRI"]) + (
                coeff_multiplier * 0.10 * s["DEF"]) + (coeff_multiplier * 0.15 * s["SHO"]),
        "CDM": lambda s: (coeff_multiplier * 0.30 * s["DEF"]) + (coeff_multiplier * 0.30 * s["PAS"]) + (
                coeff_multiplier * 0.30 * s["PHY"]) + (coeff_multiplier * 0.10 * s["DRI"]),
        "RM": lambda s: (coeff_multiplier * 0.35 * s["PAC"]) + (coeff_multiplier * 0.25 * s["DRI"]) + (
                coeff_multiplier * 0.25 * s["PAS"]) + (coeff_multiplier * 0.15 * s["SHO"]),
        "LM": lambda s: (coeff_multiplier * 0.35 * s["PAC"]) + (coeff_multiplier * 0.25 * s["DRI"]) + (
                coeff_multiplier * 0.25 * s["PAS"]) + (coeff_multiplier * 0.15 * s["SHO"]),
        "LB": lambda s: (coeff_multiplier * 0.30 * s["DEF"]) + (coeff_multiplier * 0.35 * s["PAC"]) + (
                coeff_multiplier * 0.10 * s["PHY"]) + (coeff_multiplier * 0.25 * s["PAS"]),
        "RB": lambda s: (coeff_multiplier * 0.30 * s["DEF"]) + (coeff_multiplier * 0.35 * s["PAC"]) + (
                coeff_multiplier * 0.10 * s["PHY"]) + (coeff_multiplier * 0.25 * s["PAS"]),
        "CB": lambda s: (coeff_multiplier * 0.45 * s["DEF"]) + (coeff_multiplier * 0.40 * s["PHY"]) + (
                coeff_multiplier * 0.15 * s["PAC"]),
    }

    results = {}
    for position, formula in formulas.items():
        ovr = formula(stats)
        results[position] = round(ovr)
        if debug:
            print(f"{position}: {results[position]}")

    if debug:
        print()

    return results



import os
import random

from PIL import Image

from fut_card_creator.utils import get_flag_image, get_club_logo, get_important_stats, get_card_stats_pos, \
    add_image_to_image, add_text_to_image, calculate_overall_ratings

current_folder = os.path.dirname(os.path.abspath(__file__))


class Player:
    positions = ["ST", "RW", "LW", "CF", "CAM", "CM", "CDM", "RM", "LM", "LB", "CB", "RB"]
    default_stats = ["PAC", "SHO", "PAS", "DRI", "DEF", "PHY"]

    def __init__(self, name, photo_path, club, nation, overall, position, stats):
        self.name = name
        self.photo_path = photo_path
        self.club = club
        self.nation = nation
        self.overall = overall
        self.position = position
        self.stats = stats

    def update_stat(self, stat, new_value):
        self.stats[stat] = new_value

    def weighted_random_choice(self, stats, weights):
        total_weight = sum(weights.values())
        random_choice = random.uniform(0, total_weight)
        cumulative_weight = 0
        for stat, weight in weights.items():
            cumulative_weight += weight
            if random_choice <= cumulative_weight:
                return stat
        return random.choice(list(stats.keys()))  # fallback

    def update_stats_based_on_overall(self, new_overall, calculate_type=3):
        overall_diff = new_overall - self.overall
        original_stats = True
        for stat in self.stats.keys():
            if stat not in self.default_stats:
                original_stats = False
        if not original_stats or self.position not in self.positions:
            calculate_type = 1
        if calculate_type == 1:
            for i in random.sample(self.stats.keys(), 3):
                self.stats[i] += random.randint(max(1, int(overall_diff / 2)), overall_diff * 2)
            for i in random.sample(self.stats.keys(), 3):
                self.stats[i] += random.randint(1, max(1, overall_diff))
        elif calculate_type == 2:
            stats_to_increase = random.sample(get_important_stats(self.position), 2)
            for stat in stats_to_increase:
                self.stats[stat] += random.randint(max(1, int(overall_diff / 2)), overall_diff * 2)
            other_stats_to_increase = random.sample(
                [stat for stat in self.stats.keys() if stat not in stats_to_increase], 2)
            for stat in other_stats_to_increase:
                self.stats[stat] += random.randint(1, max(1, overall_diff))
        elif calculate_type == 3:
            original_real_overall = calculate_overall_ratings(self.stats)[self.position]
            current_real_overall = original_real_overall

            # Create a weight mapping based on the position's formula
            position_weights = {
                "ST": {"SHO": 0.40, "PAC": 0.30, "DRI": 0.20, "PHY": 0.10},
                "RW": {"DRI": 0.35, "PAC": 0.30, "SHO": 0.25, "PAS": 0.10},
                "LW": {"DRI": 0.35, "PAC": 0.30, "SHO": 0.25, "PAS": 0.10},
                "CF": {"SHO": 0.40, "DRI": 0.30, "PAC": 0.20, "PAS": 0.10},
                "CAM": {"DRI": 0.35, "PAS": 0.35, "SHO": 0.25, "PAC": 0.05},
                "CM": {"PAS": 0.45, "DRI": 0.30, "DEF": 0.10, "SHO": 0.15},
                "CDM": {"DEF": 0.30, "PAS": 0.30, "PHY": 0.30, "DRI": 0.10},
                "RM": {"PAC": 0.35, "DRI": 0.25, "PAS": 0.25, "SHO": 0.15},
                "LM": {"PAC": 0.35, "DRI": 0.25, "PAS": 0.25, "SHO": 0.15},
                "LB": {"DEF": 0.30, "PAC": 0.35, "PHY": 0.10, "PAS": 0.25},
                "RB": {"DEF": 0.30, "PAC": 0.35, "PHY": 0.10, "PAS": 0.25},
                "CB": {"DEF": 0.45, "PHY": 0.40, "PAC": 0.15},
            }

            # Set a base weight for non-key stats
            base_weight = 0.05

            # Get the weight distribution for the position
            position_formula_weights = position_weights.get(self.position, {})

            # Initialize the weights with the base weight for all stats
            weights = {stat: base_weight for stat in self.stats.keys()}

            # Update the weights with the position-specific values
            for stat, weight in position_formula_weights.items():
                weights[stat] = weight

            while current_real_overall != original_real_overall + overall_diff:
                stat_to_change = self.weighted_random_choice(self.stats, weights)
                self.stats[stat_to_change] += 1
                self.stats[stat_to_change] = min(99, self.stats[stat_to_change])
                current_real_overall = calculate_overall_ratings(self.stats)[self.position]

    def update_overall(self, new_overall, update_stats=True):
        if update_stats:
            self.update_stats_based_on_overall(new_overall)
        self.overall = new_overall


class Card:
    def __init__(self, player, card_type):
        self.player = player
        self.card_type = card_type
        self.image = None

    def create_image(self):
        image_path = os.path.join(current_folder, f'images/card_templates/{self.card_type}.png')
        self.image = Image.open(image_path)
        font_path = os.path.join(current_folder, 'fonts/DINPro CondBold.otf')

        # Add name
        name_font_size = 95
        name_position = (356, 645)
        self.image = add_text_to_image(self.image, self.player.name.upper(), name_position, font_path, name_font_size,
                                       center=True)

        # Add club
        club_image = get_club_logo(self.player.club)
        club_position = (150, 520)
        club_size = (110, 110)
        self.image = add_image_to_image(self.image, club_image, club_position, club_size, center=True)

        # Add nation
        flag_image = get_flag_image(self.player.nation)
        flag_position = (150, 395)
        flag_size = (100, 60)
        self.image = add_image_to_image(self.image, flag_image, flag_position, flag_size, center=True)

        # Add overall
        overall_font_size = 150
        overall_position = (150, 150)
        self.image = add_text_to_image(self.image, str(self.player.overall), overall_position, font_path,
                                       overall_font_size, center=True)

        # Add position
        position_font_size = 70
        position_position = (150, 275)
        self.image = add_text_to_image(self.image, self.player.position, position_position, font_path,
                                       position_font_size, center=True)

        # Add photo
        if self.player.photo_path:
            photo = Image.open(self.player.photo_path)
            photo_size = (490, 490)
            photo_position = (450, 363)
            self.image = add_image_to_image(self.image, photo, photo_position, photo_size, center=True)

        # Add stats
        stats_text_pos, stats_num_pos = get_card_stats_pos()
        font_size = 70
        num_font_size = 75
        for stat in self.player.stats:
            text = stat
            position = stats_text_pos[text]  # x, y position of the text
            self.image = add_text_to_image(self.image, text, position, font_path, font_size)

            num = str(self.player.stats[stat])
            position = stats_num_pos[text]
            self.image = add_text_to_image(self.image, num, position, font_path, num_font_size)

    def update_stat(self, stat, new_value):
        self.player.update_stat(stat, new_value)

    def export_image(self, file_path):
        self.image.save(file_path)

    def update_overall(self, new_overall):
        self.player.update_overall(new_overall)



from setuptools import setup, find_packages

setup(
    name="fut_card_creator",
    version="1.0",
    packages=find_packages(),
    install_requires=[
        "Pillow>=8.0.0",  # Required for image processing
        "requests>=2.0.0",  # Required for API calls
        "fuzzywuzzy[speedup]>=0.18.0",  # Required for fuzzy string matching
        "python-Levenshtein>=0.12.0",  # Required for fuzzywuzzy speedup
    ],
    author="AngelFire",
    description="A package that allows you to create custom FUT cards",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/AngelFireLA/FutCardCreator",
    python_requires='>=3.6',
)
